'use strict';

import { predict, learnInputs, learnFilters } from './src/layer/convolution';

import convnet from 'convnet';

const inputs = [
  [
    [1,2,3],
    [4,5,6],
    [7,8,9]
  ]
];
const outputs = [
  [35,25,7],
  [3,-4,-5],
  [3,-7,-8]
];
const deltas = [
  [
    [5, 5, 5],
    [5, 5, 5],
    [5, 5, 5]
  ]
];
const filters = [
  [
    [-1,0,1],
    [0,0,0],
    [1,2,99]
  ],
];
const filterDeltas = [
  [5, 5, 5],
  [5, 5, 5],
  [5, 5, 5]
];
const biases = [1];
const context = {
  constants: {
    strideX: 1,
    strideY: 1,
    paddingY: 0,
    paddingX: 0,
    filterHeight: 3,
    filterWidth: 3,
    filterCount: 1,
    inputWidth: 3,
    inputHeight: 3,
    inputDepth: 1
  },
  output: {
    x: 3,
    y: 3,
    z: 1
  },
  thread: {
    x: 0,
    y: 0,
    z: 0
  }
};

function run(cb) {
  const output = [];
  for (let y = 0; y < context.output.y; y++) {
    const row = [];
    for (let x = 0; x < context.output.x; x++) {
      context.thread.y = y;
      context.thread.x = x;
      row.push(cb());
    }
    output.push(row);
  }
  return output;
}

// if (JSON.stringify(run(() => predict.call(context, inputs, filters, biases))) !== JSON.stringify(outputs)) {
//   throw new Error('output not correct');
// }

const convnetContext = translateContextForward(context, filters, biases);
const forwardV = convnet.ConvLayer.prototype.forward.call(convnetContext, translateInputs3D(inputs));
forwardV.dw = translateInputs2D([[1,2,3],[4,5,6],[7,8,9]]).w;
convnetContext.in_act.w = convnetContext.in_act.w.map((_, i) => 0);
convnetContext.out_act.w = translateInputs2D([[1,2,3],[4,5,6],[7,8,9]]).w;
convnetContext.filters[0].w = translateInputs2D([[1,2,3],[4,5,6],[7,8,9]]).w;
const backV = convnet.ConvLayer.prototype.backward.call(convnetContext);
console.log('test');

console.log(JSON.stringify(run(() => learnInputs.call(context, [[[1,2,3],[4,5,6],[7,8,9]]], [[[1,2,3],[4,5,6],[7,8,9]]], [[[1,2,3],[4,5,6],[7,8,9]]]))));
console.log(JSON.stringify(vDwToArray(convnetContext.in_act)));


function vDwToArray(v) {
  const output = [];
  for (let y = 0; y < v.sy; y++) {
    const row = [];
    for (let x = 0; x < v.sx; x++) {
      row.push(v.get_grad(x, y, 0));
    }
    output.push(row);
  }

  return output;
}

function translateContextForward(context, filters, biases) {
  filters = translateInputs3D(filters);
  return {
    out_sx: context.output.x,
    out_sy: context.output.y,
    out_depth: context.output.z,
    stride: context.constants.strideX,
    pad: context.constants.paddingX,
    filters: [filters],
    biases: translateInputs1D(biases)
  }
}

function translateInputs3D(inputs) {
  const depth = inputs.length;
  const height = inputs[0].length;
  const width = inputs[0][0].length;
  const vol = new convnet.Vol(width, height, depth, 0);
  for (let z = 0; z < depth; z++) {
    for (let y = 0; y < height; y++) {
      for (let x = 0; x < width; x++) {
        vol.set(x, y, z, inputs[z][y][x]);
      }
    }
  }
  return vol;
}

function translateInputs2D(inputs) {
  const height = inputs.length;
  const width = inputs[0].length;
  const vol = new convnet.Vol(width, height, 1, 0);
  for (let y = 0; y < height; y++) {
    for (let x = 0; x < width; x++) {
      vol.set(x, y, 0, inputs[y][x]);
    }
  }
  return vol;
}

function translateInputs1D(inputs) {
  const width = inputs.length;
  const vol = new convnet.Vol(width, 1, 1, 0);
  for (let x = 0; x < width; x++) {
    vol.set(x, 0, 0, inputs[x]);
  }
  return vol;
}