'use strict';

import gpuMock from 'gpu-mock.js';
import { predict, learnInputs, learnFilters, learnBiases } from './src/layer/fully-connected';

import convnet from 'convnet';

function getFilters() {
  const filter = new convnet.Vol(9, 1, 1);
  for (let x = 0; x < 9; x++) {
    filter.set(x, 0, 0, x);
  }
  return filter;
}

function getInputs() {
  const inputs = new convnet.Vol(9, 1, 1);
  let val = 0;
  for (let x = 0; x < 3; x++) {
    for (let y = 0; y < 3; y++) {
      inputs.set(val, 0, 0, val++);
    }
  }
  return inputs;
}

function getBiases() {
  const biases = new convnet.Vol(9, 1, 1);
  for (let x = 0; x < 9; x++) {
    biases.set(x, 0, 0, x);
  }
  return biases;
}

const context = {
  out_depth: 9,
  num_inputs: 9,
  in_act: getInputs(),
  filters: [getFilters(), getFilters(), getFilters(), getFilters(), getFilters(), getFilters(), getFilters(), getFilters(), getFilters()],
  biases: getBiases()
};
convnet.FullyConnLayer.prototype.forward.call(context, getInputs());
context.out_act.dw = context.out_act.dw.map((val, i) => i);
convnet.FullyConnLayer.prototype.backward.call(context);

console.log(context.in_act.dw);
process.exit(0);
const results = gpuMock(predict, {
  output: [9],
  constants: {
    inputDepth: 1,
    inputHeight: 1,
    inputWidth: 9
  }
})([[0,1,2,3,4,5,6,7,8]], [
  [0,1,2,3,4,5,6,7,8],
  [0,1,2,3,4,5,6,7,8],
  [0,1,2,3,4,5,6,7,8],
  [0,1,2,3,4,5,6,7,8],
  [0,1,2,3,4,5,6,7,8],
  [0,1,2,3,4,5,6,7,8],
  [0,1,2,3,4,5,6,7,8],
  [0,1,2,3,4,5,6,7,8],
  [0,1,2,3,4,5,6,7,8]
], [0,1,2,3,4,5,6,7,8]);
console.log(results);

function translateContextForward(context, filters, biases) {
  filters = translateInputs3D(filters);
  return {
    out_sx: context.output.x,
    out_sy: context.output.y,
    out_depth: context.output.z,
    stride: context.constants.strideX,
    pad: context.constants.paddingX,
    filters: [filters],
    biases: translateInputs1D(biases)
  }
}